#!/bin/bash
#
# http:/www.ultimatebootcd.com/
#
# This script edits the syslinux.cfg file from pmagic so to merge it into UBCD.
#
# bash tutorial: http://tldp.org/LDP/abs/html/
# sed tutorial:  http://www.grymoire.com/Unix/Sed.html
#
# THIS FILE SHALL USE UNIX-LIKE NEWLINES
#

# pwd_msys prints the working directory in Windows format under Windows.
# It is used in this script for displaying messages containing paths.
# The '-W' switch in pwd works with MSYS Bash only.
# It shall not be stored in variables since
# the Windows paths cannot be interpreted in unix shells.
pwd_msys () {
    if [ "X${OSTYPE}" = 'Xmsys' ]; then
        pwd -W | sed -e 's|/|\\|g';
    else
        pwd
    fi
}

echo
echo
echo 'PM2UBCD: Merge PMagic into UBCD'
echo '_______'
echo
echo

# Define ISO filename (can be overridden by passing arguments to the script)
# To reduce problems, it is better to only use eight ISO9660 characters and
# no spaces.
# This corresponds to the "iso_filename=[PATH/TO/]filename.iso" boot code
# in PMagic.
ISO_NAME=UBCD525_custom.iso

# Change the working directory to the root directory of the extracted UBCD iso
# where this pm2ubcd script exists.
# The script should be executed starting from its own dir.
# The reason to have, nevertheless, the path of "UBCD_extracted" as argument is
# for future potential enhancements.
#
# POTENTIAL OPTIONAL ALTERNATIVE - not tested:
# --------------------------------------------
# Change working directory to dir from which we started
# CWD=`echo "${PWD}"`
# SCRIPTFILE="$(dirname -- "$0")"
# SCRIPTFILE="$(realpath "$SCRIPTFILE" 2>/dev/null || echo $SCRIPTFILE)"
# cd "$SCRIPTFILE"
# --------------------------------------------
#
SCRIPTFILE="$(cd "${0%/*}" 2>/dev/null; echo "$PWD"/"${0##*/}")"
SCRIPTDIR=`dirname -- "$SCRIPTFILE"`
cd "$SCRIPTDIR"
ls '../../../../ubcd/tools/linux/pm2ubcd/pm2ubcd.sh' > /dev/null 2>&1
if [ "$?" -eq "0" ]; then
    cd "${SCRIPTDIR}/../../../../"
    UBCD_ROOT=`echo "${PWD}"`
else
    echo "PM2UBCD ERROR: Please execute the script from inside its own UBCD directory:"
    echo '       (./ubcd/tools/linux/pm2ubcd/pm2ubcd.sh)'
    exit
fi

# Check how many arguments are passed to the script
case "${#@}" in
    0)
        echo
        if [ "X${OSTYPE}" = 'Xmsys' ]; then
            # Define default PMagic extracted root for Windows OS
            PMAGIC_ROOT='c:\pmagic_extracted'
            echo '  USAGE:   pm2ubcd.cmd [extracted_pmagic [extracted_ubcd [iso_filename]]]'
            echo
            echo '  Example: pm2ubcd c:\pmagic_extracted c:\ubcd_extracted ubcd525_custom.iso'
            echo
            echo "             PMAGIC extracted to 'c:\pmagic_extracted',"
            echo "             UBCD extracted to 'c:\ubcd_extracted',"
        else
            PMAGIC_ROOT='~/pmagic_extracted/'
            echo '  USAGE:   pm2ubcd.sh [extracted_pmagic [extracted_ubcd [iso_filename]]]'
            echo
            echo '  Example: pm2ubcd.sh ~/pmagic_extracted/ ~/ubcd_extracted/ ubcd525_custom.iso'
            echo
            echo "             PMAGIC extracted to '~/pmagic_extracted/',"
            echo "             UBCD extracted to '~/ubcd_extracted/',"
        fi
        echo "             ISO filename 'ubcd525_custom.iso'"
        echo
        echo "  ___________________________________________________"
        echo "  Paths shall be limited to US-ASCII characters only."
        echo "  It is recommended to use paths with NO spaces."
        echo "  ___________________________________________________"
        echo
        echo "  PM2UBCD: No arguments were provided."
        echo "  You must specify at least the path to which PMAGIC is extracted or mounted."
        echo
#       " The following E_X_I_T command is temporal until further development."
        exit
        ;;
    1)
        # Assign the respective paths.
        # Avoid using the problematic backslash character, just in case it gets here.
        PMAGIC_ROOT=`echo $1 | sed -e 's|[\\]|/|g'`
        ;;
    2)
        PMAGIC_ROOT=`echo $1 | sed -e 's|[\\]|/|g'`
        UBCD_ROOT=`echo $2 | sed -e 's|[\\]|/|g'`
        ;;
    3)
        PMAGIC_ROOT=`echo $1 | sed -e 's|[\\]|/|g'`
        UBCD_ROOT=`echo $2 | sed -e 's|[\\]|/|g'`
        ISO_NAME=`echo $3 | sed -e 's|[\\]|/|g'`
        ;;
esac

# Expand the '~' character in paths valid for bash or dash
if [ `echo ${PMAGIC_ROOT} | sed -e 's/^\(.\).*$/\1/'` = "~" ]; then
    PMAGIC_ROOT=`echo "echo ${PMAGIC_ROOT}" | bash`
fi
if [ `echo ${UBCD_ROOT} | sed -e 's/^\(.\).*$/\1/'` = "~" ]; then
    UBCD_ROOT=`echo "echo ${UBCD_ROOT}" | bash`
fi

echo

if [ -d "${PMAGIC_ROOT}" ]; then
    PMAGIC_ROOT=`cd "${PMAGIC_ROOT}" && echo "${PWD}"`
else
    echo
    echo "PM2UBCD ERROR:"
    echo " \"${PMAGIC_ROOT}\" "
    echo "is not a directory."
    PMAGIC_ROOT=''
    echo
    exit 1
fi

if [ -d "${UBCD_ROOT}" ]; then
    UBCD_ROOT=`cd "${UBCD_ROOT}" && echo "${PWD}"`
else
    echo
    echo "PM2UBCD ERROR:"
    echo " \"${UBCD_ROOT}\" "
    echo "is not a directory."
    UBCD_ROOT=''
    echo
    exit 1
fi

if [ ! -f "${UBCD_ROOT}/ubcd/tools/linux/pm2ubcd/pm2ubcd.sed" ]; then
    echo
    echo "PM2UBCD ERROR: pm2ubcd.sed is not found in"
    echo "              `( cd "${UBCD_ROOT}/ubcd/tools/linux/pm2ubcd" && pwd_msys ; )`"
    echo
    exit 1
fi

# iso_filename is supposed to end with .iso (preferable) or with .ISO.
# Other combinations, like .Iso for example, are not checked in this section.
# In other words, don't be lazy and type-in the complete iso_filename.
# Note: The optional path to iso_filename shall not contain spaces.
#
ISO_NAME_EXT=`echo "${ISO_NAME}" | sed -e 's|^..*\(....\)$|\1|'`
#
# The following are two alternative and theoretically equivalent
# conditional expressions for ISO_NAME_EXT.
# This first alternative does not need an "else" statement
# but it might not work correctly in some systems.
# if [ "X${ISO_NAME_EXT}" != "X.iso" ] && [ "X${ISO_NAME_EXT}" != "X.ISO" ]
# then
#     ISO_NAME=`echo "${ISO_NAME}.iso"`
# fi
#
# This second alternative could work better in some systems.
if [ "X${ISO_NAME_EXT}" = "X.iso" ] || [ "X${ISO_NAME_EXT}" = "X.ISO" ]
then
    # Leaving iso_filename without changes.
    ISO_NAME=`echo "${ISO_NAME}"`
else
    ISO_NAME=`echo "${ISO_NAME}.iso"`
fi
#
ISO_NAME_EXT=''

echo
echo "PATH_TO_EXTRACTED_PMAGIC_ISO : `( cd "${PMAGIC_ROOT}" && pwd_msys ; )`"
echo
echo "PATH_TO_EXTRACTED_UBCD_ISO   : `( cd "${UBCD_ROOT}" && pwd_msys ; )`"
echo
echo "ISO_FILE_NAME                : `( echo ${ISO_NAME} ; )`"
echo

# Get new sqfs file name, which may be useful later.
# Old versions of PMagic use lowercase 'sqfs' names.
# The PXE variant of PMagic doesn't have a separated SQFS file.
#
# The following directory shall exist so we FIRST go to it as fallback.
cd "${PMAGIC_ROOT}"
# The following directory should exist in std PMAGIC
# but maybe it does not exist if customized.
if [ -d "${PMAGIC_ROOT}/pmagic/pmodules/" ]; then
    cd "${PMAGIC_ROOT}/pmagic/pmodules/"
    PMAGIC_SQFS_NAME=`find * -type f -a \( -name '*.SQFS' -o -name '*.sqfs' \) | sed -n -e '1 p'`
    if [ "X$PMAGIC_SQFS_NAME" = "X" ]; then
        echo
        echo "PM2UBCD: No SQFS file is found in"
        echo "        `( cd "${PMAGIC_ROOT}/pmagic/pmodules/" && pwd_msys ; )`"
    else
        PMAGIC_VERSION=`echo $PMAGIC_SQFS_NAME | \
                        sed -e 's/.*PMAGIC_\(.*\)\.SQFS/\1/g' -e 's/.*pmagic-\(.*\)\.sqfs/\1/g'`
        echo "NEW_PMAGIC_VERSION           : `( echo ${PMAGIC_VERSION} ; )`"
    fi
fi
echo

# Ask for confirmation.
# There are many alternative methods to achieve this.
echo
read -p "Continue (y/n)? Press Y to continue with these values " -n 1 -r REPLY
echo
echo
if [ "X$REPLY" = "XY" ] || [ "X$REPLY" = "Xy" ]
then
    echo "PM2UBCD STARTING MOVEMENTS ..... "
    echo
else
    echo "You do not want to continue with these values so we are done."
    echo
    echo
    cd "${UBCD_ROOT}/ubcd/tools/linux/pm2ubcd"
    exit
fi

# Is the destination media writable\?
# For the following test to work as expected the pm2ubcd.tst file should not exist.
cd "${UBCD_ROOT}"
if [ -e "pm2ubcd.tst" ]; then
    echo
    echo 'PM2UBCD ERROR: Detected a \"pm2ubcd.tst\" file.'
    echo 'Another PM2UBCD script may be processing this directory.'
    echo 'Exiting for safety.'
    echo
    cd "${UBCD_ROOT}/ubcd/tools/linux/pm2ubcd"
    exit 1
fi
# The UBCD ROOT directory tree shall be writable and located in a writable media.
echo "This file tests whether the directory is writable." >> pm2ubcd.tst 2>/dev/null
if [ ! -e "pm2ubcd.tst" ]; then
    echo
    echo "PM2UBCD ERROR: The media on which"
    ( pwd_msys ; )
    echo "is located is not writable."
    echo
    cd "${UBCD_ROOT}/ubcd/tools/linux/pm2ubcd"
    exit 1
fi
# Now there is a pm2ubcd.tst file so before any following exit
# the file needs to be deleted so to indicate that the script has finished.

# Get old PMAGIC_YYYY_MM_DD.SQFS files from ./pmagic/pmodules/ to be deleted.
mkdir -p "${UBCD_ROOT}/pmagic/pmodules/"
cd "${UBCD_ROOT}/pmagic/pmodules/"
echo
SQUASH_FILE=''
SQUASH_FILES=''
# The following line is valid until PMAGIC_2019_12_31.
SQUASH_FILES=`find . -type f -a \( -name 'PMAGIC_201*.SQFS' -o -name 'pmagic-*.sqfs' \)`
# " Removing old squashfs files. INTERACTIVE"
for SQUASH_FILE in $SQUASH_FILES; do
    echo
    echo "We are now in"
    ( pwd_msys ; )
    echo
    echo "Answer Y and press ENTER to accept the following action : "
    echo
    rm -i "${SQUASH_FILE}"
    echo
done
SQUASH_FILE=''
SQUASH_FILES=''

# Get all syslinux config files from ./pmagic/boot/syslinux/ that need backup.
mkdir -p "${UBCD_ROOT}/pmagic/boot/syslinux/"
cd "${UBCD_ROOT}/pmagic/boot/syslinux/"
SYSLINUX_CFG_PMAGIC=''
SYSLINUX_CFGS_PMAGIC=''
SYSLINUX_CFGS_PMAGIC=`find . -type f -a \( -name '*cfg*' -not \( -name '*.old' \) \)`
# Back up cfg files. SEMI-INTERACTIVE.
echo
echo 'Backup cfg file(s) in'
( pwd_msys ; )
for SYSLINUX_CFG_PMAGIC in $SYSLINUX_CFGS_PMAGIC; do
    echo "${SYSLINUX_CFG_PMAGIC}"
    mv -i -T "${SYSLINUX_CFG_PMAGIC}" "${SYSLINUX_CFG_PMAGIC}.old"
done
SYSLINUX_CFG_PMAGIC=''
SYSLINUX_CFGS_PMAGIC=''
#    The alternative of moving instead of copying simplifies the correct
#    selection of cfg files to be edited with sed
#    but if the script is executed more than once then the "old" backup files
#    are overwritten and in-place edition is not possible as it requires for
#    the existence of cfg files in "pmagic_extracted/boot/syslinux/" and to be
#    copied over to "ubcd_extracted/boot/syslinux/".
#
#    Alternative copy instead of move.
#    Copying affects the selection of cfg files that will be edited with sed
#    so the script can be executed in-place but it may cause a wrong selection
#    of cfg files to be edited with sed. This alternative has not been tested.
#    tar -c "${SYSLINUX_CFG_PMAGIC}" | ( cd "${UBCD_ROOT}/pmagic/boot/" && tar -xvBf - && mv -i -T "${SYSLINUX_CFG_PMAGIC}" "${UBCD_ROOT}/pmagic/boot/syslinux/${SYSLINUX_CFG_PMAGIC}.old" ; )

# Copying _content_ of "./pmagic/" directory
#  and _content_ of its subdirs.
# First get list of files.
# {UBCD_ROOT}/pmagic/ already exists.
# The following directory shall exist so we FIRST go to it as fallback.
cd "${PMAGIC_ROOT}"
# The following directory should exist in std PMAGIC
# but maybe it does not exist if customized.
if [ -d "${PMAGIC_ROOT}/pmagic/" ]; then
    cd "${PMAGIC_ROOT}/pmagic/"
    PMAGICDIR_FILE=''
    PMAGICDIR_FILES=''
    PMAGICDIR_FILES=`find . -type f`
    echo
    echo Copying content of
    ( pwd_msys ; )
    echo "to the UBCD respective location."
    echo "This may take some time."
    for PMAGICDIR_FILE in $PMAGICDIR_FILES; do
        tar -c "${PMAGICDIR_FILE}" | ( cd "${UBCD_ROOT}/pmagic/" && tar -xvBf - ; )
    done
    PMAGICDIR_FILE=''
    PMAGICDIR_FILES=''
fi

# Copying "./boot/" directory and subdirs structure
# except "./syslinux/" dir.
# The following directory shall exist so we FIRST go to it as fallback.
cd "${PMAGIC_ROOT}"
# The following directory should exist in std PMAGIC
# but maybe it does not exist if customized.
if [ -d "${PMAGIC_ROOT}/boot/" ]; then
    cd "${PMAGIC_ROOT}/boot/"
    BOOTDIR_FILE=''
    BOOTDIR_FILES=''
    BOOTDIR_FILES=`find . -type f -not \( -path './syslinux/*' \)`
    echo
    echo Copying content of
    ( pwd_msys ; )
    echo "to the UBCD respective location."
    for BOOTDIR_FILE in $BOOTDIR_FILES; do
        tar -c "${BOOTDIR_FILE}" | ( cd "${UBCD_ROOT}/pmagic/boot/" && tar -xvBf - ; )
    done
    BOOTDIR_FILE=''
    BOOTDIR_FILES=''
fi

# Copying _content_ of "./boot/syslinux/" directory.
# The following directory shall exist so we FIRST go to it as fallback.
cd "${PMAGIC_ROOT}"
# The following directory should exist in std PMAGIC
# but maybe it does not exist if customized.
if [ -d "${PMAGIC_ROOT}/boot/syslinux/" ]; then
    cd "${PMAGIC_ROOT}/boot/syslinux/"
    SYSLINUXDIR_FILE=''
    SYSLINUXDIR_FILES=''
    SYSLINUXDIR_FILES=`find . -type f -not \( -name 'iso*' \) -not \( -name '*.cat*' \) \
                              -not \( -name 'memdisk' \) -not \( -name '*.bin' \) \
                              -not \( -name '*.c32' \) -not \( -name '*efi*' \)`
    echo
    echo Copying content of
    ( pwd_msys ; )
    echo "to the UBCD respective location."
    for SYSLINUXDIR_FILE in $SYSLINUXDIR_FILES; do
        tar -c "${SYSLINUXDIR_FILE}" | ( cd "${UBCD_ROOT}/pmagic/boot/syslinux/" && tar -xvBf - ; )
    done
    SYSLINUXDIR_FILE=''
    SYSLINUXDIR_FILES=''
fi

echo
echo "PM2UBCD: Starting edition of cfg files ....."
echo

# Get all syslinux config files from ./pmagic/boot/syslinux/ that need edition.
cd "${UBCD_ROOT}/pmagic/boot/syslinux"
SYSLINUX_CFG_PMAGIC=''
SYSLINUX_CFGS_PMAGIC=''
SYSLINUX_CFGS_PMAGIC=`find . -type f -a \( -name '*cfg*' -not \( -name '*.old' \) \)`
( pwd_msys ; )
    # 1. Change paths from "/boot/" to "/pmagic/boot/" where appropriate.
    # 2. Add iso_filename parameter. Requires ISO_NAME variable.
    # 3. Convert config files in /pmagic/boot/syslinux/ to text files with DOS
    #    line endings.
    # Note: it is necessary to enclose '/' in square brackets for bash/Win32
    for SYSLINUX_CFG_PMAGIC in $SYSLINUX_CFGS_PMAGIC; do
        echo "${SYSLINUX_CFG_PMAGIC}"
        sed -f "${UBCD_ROOT}/ubcd/tools/linux/pm2ubcd/pm2ubcd.sed" < "${SYSLINUX_CFG_PMAGIC}" | \
        sed -e "s|\(^[ ]*APPEND .*\<vmalloc[\=][0-9]\{1,\}MiB\>\)|\1 iso_filename=\'${ISO_NAME}\'|" | \
        sed -e 's|[\ ]*\r*$||g' | \
        sed -e 's|[\ ]*$|\r|g' > "${SYSLINUX_CFG_PMAGIC}.tmp"
        mv -f "${SYSLINUX_CFG_PMAGIC}.tmp" "${SYSLINUX_CFG_PMAGIC}"
    done
    # Other alternative possible sed expressions with similar results:
    #    sed -e 's|^ *F[[:digit:]] \(.*\)\message\(.*\)\.txt|^F[[:digit:]] /pmagic/boot/syslinux/message\2\.txt'
    # In case there are multiple comma-separated initrd values for PMAGIC
    # then the iso_filename parameter step above should be changed to a more generic statement as follows:
    #    sed -e "s|\(initrd=[/]pmagic[/]initrd\.img[^\r\n]*\)$|\1 iso_filename=\'${ISO_NAME}\'|" | \

echo
echo
echo "PM2UBCD: Done."
echo
echo
echo "        Optionally, you can manually delete possible remnants like:"
echo '         "old" cfg file(s) in'
( cd "${UBCD_ROOT}/pmagic/boot/syslinux/" && pwd_msys ; )
echo "         and the original"
( cd "${PMAGIC_ROOT}" && pwd_msys ; )
echo "         directory if are not required anymore."
echo
echo

cd "${UBCD_ROOT}"
rm -f pm2ubcd.tst

cd "${UBCD_ROOT}/ubcd/tools/linux/pm2ubcd"
exit 0
